package Weather::Com::L10N;

use 5.006;
use Carp;
use base 'Locale::Maketext';

# have a cvs driven version...
our $VERSION = sprintf "%d.%03d", q$Revision: 1.3 $ =~ /(\d+)/g;

%Lexicon = ( '_AUTO' => 1, );    


1;